==> react-code-highlight-0.0.1.tgz: package/src/components/index.tsx <==
import React, { useRef, useEffect, useState } from 'react';
import styles from './index.module.scss';
import hightlight from 'highlight.js';
import 'highlight.js/styles/github.css';

type props = {
  language?: string,
  debouneTime?: number,
  cb?: (content: string) => any,
  className?: string,
}

const TAB = 'Tab';

export const HighLighter = ({ debouneTime = 666, language = 'javascript', cb = () => { }, className = '' }: props) => {
  const [content, setContent] = useState('');

  const refTimer = useRef<ReturnType<typeof setTimeout>>();
  useEffect(() => {
    refTimer.current && clearTimeout(refTimer.current);
    refTimer.current = setTimeout(() => cb(content), debouneTime);
  }, [content, refTimer, debouneTime, cb])

  const ref = useRef<HTMLElement>(null);
  useEffect(() => {
    const target = ref.current;
    if (!target) { return }
    hightlight.highlightBlock(target);
  }, [content, ref])

  const refTextArea = useRef<HTMLTextAreaElement>(null);
  const [endPos, setEndPos] = useState(0);
  useEffect(() => {
    const target = refTextArea.current;
    if (!target) { return }
    target.setSelectionRange(endPos, endPos);
  }, [refTextArea, endPos])

  const keydown = (e: React.KeyboardEvent<HTMLTextAreaElement>) => {
    if (!refTextArea.current) { return }
    if (e.key === TAB) {
      e.preventDefault();
      const selStartPos = e.currentTarget.selectionStart;
      const selEndPos = e.currentTarget.selectionEnd;
      const tapSpace = selStartPos === selEndPos
        ? new Array(4).fill(' ').join('')
        : ' ';
      const value = content.substring(0, selStartPos)
        + tapSpace
        + content.substring(selEndPos, content.length);
      setContent(value);
      setEndPos(selStartPos + tapSpace.length);
    }
  }

  const refPre = useRef<HTMLPreElement>(null);
  const scroll = (e: React.WheelEvent<HTMLTextAreaElement>) => {
    if (!refPre.current) {
      return;
    }
    refPre.current.scrollTo(0, e.currentTarget.scrollTop)
  }

  return <div className={`${styles.container} ${className}`}>
    <textarea
      ref={refTextArea}
      className={styles.input}
      value={content}
      spellCheck='false'
      onChange={(e) => setContent(e.currentTarget.value)}
      onKeyDown={keydown}
      onScroll={scroll}
    />
    <pre ref={refPre} className={styles.output}>
      <code className={language} ref={ref}>
        {content}
      </code>
    </pre>
  </div>
}

==> react-code-highlight-0.0.7.tgz: package/src/components/index.tsx <==
import React, { useRef, useEffect, useState } from 'react';
import styles from './index.module.scss';
import hightlight from 'highlight.js';
import 'highlight.js/styles/github.css';

type props = {
  language?: string,
  debouneTime?: number,
  cb?: (content: string) => any,
  className?: string,
}

const TAB = 'Tab';

export const HighLighter = ({ debouneTime = 666, language = 'javascript', cb = () => { }, className = '' }: props) => {
  const [content, setContent] = useState('');

  const refTimer = useRef<ReturnType<typeof setTimeout>>();
  useEffect(() => {
    refTimer.current && clearTimeout(refTimer.current);
    refTimer.current = setTimeout(() => cb(content), debouneTime);
  }, [content, refTimer, debouneTime, cb])

  const ref = useRef<HTMLElement>(null);
  useEffect(() => {
    const target = ref.current;
    if (!target) { return }
    hightlight.highlightElement(target);
  }, [content, ref])

  const refTextArea = useRef<HTMLTextAreaElement>(null);
  const [endPos, setEndPos] = useState(0);
  useEffect(() => {
    const target = refTextArea.current;
    if (!target) { return }
    target.setSelectionRange(endPos, endPos);
  }, [refTextArea, endPos])

  const keydown = (e: React.KeyboardEvent<HTMLTextAreaElement>) => {
    if (!refTextArea.current) { return }
    if (e.key === TAB) {
      e.preventDefault();
      const selStartPos = e.currentTarget.selectionStart;
      const selEndPos = e.currentTarget.selectionEnd;
      const tapSpace = selStartPos === selEndPos
        ? new Array(4).fill(' ').join('')
        : ' ';
      const value = content.substring(0, selStartPos)
        + tapSpace
        + content.substring(selEndPos, content.length);
      setContent(value);
      setEndPos(selStartPos + tapSpace.length);
    }
  }

  const refPre = useRef<HTMLPreElement>(null);
  const scroll = (e: React.WheelEvent<HTMLTextAreaElement>) => {
    if (!refPre.current) {
      return;
    }
    refPre.current.scrollTo(0, e.currentTarget.scrollTop)
  }

  return <div className={`${styles.container} ${className}`}>
    <textarea
      ref={refTextArea}
      className={styles.input}
      value={content}
      spellCheck='false'
      onChange={(e) => setContent(e.currentTarget.value)}
      onKeyDown={keydown}
      onScroll={scroll}
    />
    <pre ref={refPre} className={styles.output}>
      <code className={language} ref={ref}>
        {content}
      </code>
    </pre>
  </div>
}
